#!/usr/bin/env python3
"""
@author: RealStickman
"""
import configparser
import os
import subprocess
import tkinter as tk
import sys
from pathlib import Path
from tkinter import messagebox, filedialog, ttk
import praw
import prawcore
from multiprocessing import Process

# variables
import progvars
# modules
import modules

import grab

def addfr():
    #remove add/remove subreddit buttons
    butaddsub.destroy()
    butremsub.destroy()

    #Frame for adding subs
    global addsub
    addsub = ttk.Frame(normal)
    addsub.grid(row=1, column=0, sticky="W")

    #Label
    ttk.Label(addsub, text="Enter subreddit:").grid(row=0, column=0, sticky="W")

    #textbox
    global txtent
    txtent = ttk.Entry(addsub)
    txtent.grid(row=1, column=0, sticky="W")

    #abort
    ttk.Button(addsub, text="Cancel", command=cancel).grid(row=2, column=0, sticky="W")

    #try to add
    ttk.Button(addsub, text="Add", command=save).grid(row=2, column=0, sticky="E")

#kills the add subreddit window
def cancel():
    addsub.destroy()

    # add new subreddits
    global butaddsub
    butaddsub = ttk.Button(normal, text="+", command=addfr)
    butaddsub.grid(row=1, column=0, sticky="W", padx=2)

    # remove subreddits
    global butremsub
    butremsub = ttk.Button(normal, text="-", command=listbox)
    butremsub.grid(row=1, column=0, sticky="E")

#for saving subreddits
def save():
    #reddit stuff
    reddit = praw.Reddit(client_id="48VCokBQkKDsEg",
                         client_secret=None,
                         user_agent="grab, a Reddit download bot by /u/RealStickman_")

    #hope the user wrote something
    try:
        txte = txtent.get()
        #check if what the user wrote makes sense
        try:
            subreddit = reddit.subreddit(txte)
            subreddit.title

            #read the config
            modules.readconf()
            #add entered subreddit
            progvars.sublist += subreddit.display_name + ";"
            #that's were we're gonna put it
            modules.writeconf()
            print("Added " + subreddit.display_name)

            #reformatting list of subs
            progvars.sublf = progvars.sublist.split(";")
            labelsublist = [x for x in progvars.sublf if x]
            # displays list of subreddits
            lbox.delete(0, 'end')
            for i in range(len(labelsublist)):
                lbox.insert('end', labelsublist[i])

            #destroy window
            addsub.destroy()
        #if the subreddit can't be found
        except prawcore.exceptions.Redirect:
            messagebox.showerror("Wrong Subreddit", subreddit.display_name + " does not exist")
        # add new subreddits
        global butaddsub
        butaddsub = ttk.Button(normal, text="+", command=addfr)
        butaddsub.grid(row=1, column=0, sticky="W", padx=2)

        # remove subreddits
        global butremsub
        butremsub = ttk.Button(normal, text="-", command=listbox)
        butremsub.grid(row=1, column=0, sticky="E")
    #if the user didn't type anything
    except TypeError:
        messagebox.showerror("No Subreddit", "Please fill in a subreddit")

#closes the window and stops the program
def stop():
    root.destroy()
    sys.exit()
    exit()

# showing info message and have it be interactable
def runprog():
    print("works runprog")
    messagebox.showinfo("Running", "Program is running")
    main.update_idletasks()

# process that executes the download
def rundl():
    print("works rundl")
    # to make sure the number of subs is correct
    modules.readconf()
    # download images from reddit
    modules.multiprocdl()

# runs the program
def run():
    # set up processes
    procprog = Process(target = runprog())
    procrundl = Process(target = rundl())

    # start processes
    procrundl.start()
    procprog.start()

    # make sure both processes are finished
    procprog.join()
    procrundl.join()

    messagebox.showinfo("Finished", "Program has finished")

def listbox():
    result = tk.messagebox.askyesno("Delete", "Remove selected subreddit from list?", icon='warning')
    if result == True:
        #read the config
        modules.readconf()
        selec = lbox.curselection()
        val = lbox.get(selec)
        global sublist
        progvars.sublist = progvars.sublist.replace(";" + val, '')
        modules.writeconf()
        print("Deleted " + val)
        updatlist()
    else:
        print("Nothing deleted")

def updatlist():
    #read the config
    modules.readconf()
    lcusubl = progvars.sublist.split(";")
    labsubl = [x for x in lcusubl if x]
    lbox.delete(0, 'end')
    for i in range(len(labsubl)):
        lbox.insert(0, labsubl[i])

def catsel():
    modules.readconf()
    #sets category based on the radiobutton selected
    if catvar.get() == 1:
        progvars.category = "controversial"
        modules.writeconf()
        print("Setting category to controversial")
    # elif catvar.get() == 2:
    #     progvars.category = "gilded"
    #     writeconf()
    #     print("Setting category to gilded")
    elif catvar.get() == 3:
        progvars.category = "hot"
        modules.writeconf()
        print("Setting category to hot")
    elif catvar.get() == 4:
        progvars.category = "new"
        modules.writeconf()
        print("Setting category to new")
    elif catvar.get() == 5:
        progvars.category = "rising"
        modules.writeconf()
        print("Setting category to rising")
    elif catvar.get() == 6:
        progvars.category = "top"
        modules.writeconf()
        print("Setting category to top")
    else:
        print("Something went wrong")

#save the limit
def savelim():
    modules.readconf()
    print("New limit: " + limspin.get())
    global lim
    progvars.lim = limspin.get()
    modules.writeconf()

#expands the window to show all settings
def winexp():
    global extend
    extend = ttk.Frame(main)
    extend.grid(row=0, column=2, sticky="W")
    modules.readconf()

    #destroy button to expand window
    butexp.destroy()

    #other settings
    setfr = ttk.Frame(extend)
    setfr.grid(row=0, column=1, sticky="W")

    #change theme
    ttk.Label(setfr, text="Change theme").grid(row=0, column=0, sticky="W")
    ttk.Button(setfr, text="light", command=setlight).grid(row=1, column=0, sticky="E")
    ttk.Button(setfr, text="dark", command=setdark).grid(row=1, column=1, sticky="W")

    #frame for various settings
    global varfr
    varfr = ttk.Frame(extend)
    varfr.grid(row=1, column=0, sticky="W")

    # Path
    global pathlab
    pathlab = ttk.Label(varfr, text="Path: " + progvars.path)
    pathlab.grid(row=0, column=1, sticky="W")
    global butpath
    butpath = ttk.Button(varfr, text="...", command=dirdialog)
    butpath.grid(row=0, column=2, sticky="W")

    # Limit
    global limspin
    limspin = tk.Spinbox(varfr, from_=0, to=100, width=5, bg=background, fg=foreground)
    limspin.grid(row=1, column=1, sticky="W")
    # show currently set limit
    i = 0
    while i < progvars.lim:
        limspin.invoke(element="buttonup")
        i += 1

    # limit save button
    global butsave
    butsave = ttk.Button(varfr, text="Save", command=savelim)
    butsave.grid(row=1, column=1, sticky="W", padx="55")

    #frame for categories
    global categ
    categ = ttk.Frame(extend)
    categ.grid(row=0, column=0, sticky="W")
    # Category label
    global labcateg
    labcateg = ttk.Label(categ, text="Category:")
    labcateg.grid(row=0, column=0, sticky="W")

    # category buttons for selection
    global catvar
    catvar = tk.IntVar()

    global catcont
    catcont = tk.Radiobutton(categ, text="controversial", variable=catvar, value=1, command=catsel, bg=background, fg=foreground, relief="ridge")
    catcont.grid(row=1, column=0, sticky="W")

    # global catgil
    # catgil = tk.Radiobutton(categ, text="gilded", variable=catvar, value=2, command=catsel, bg=background, fg=foreground, relief="ridge")
    # catgil.grid(row=1, column=1, sticky="W")

    global cathot
    cathot = tk.Radiobutton(categ, text="hot", variable=catvar, value=3, command=catsel, bg=background, fg=foreground, relief="ridge")
    cathot.grid(row=1, column=2, sticky="W")

    global catnew
    catnew = tk.Radiobutton(categ, text="new", variable=catvar, value=4, command=catsel, bg=background, fg=foreground, relief="ridge")
    catnew.grid(row=2, column=0, sticky="W")

    global catris
    catris = tk.Radiobutton(categ, text="rising", variable=catvar, value=5, command=catsel, bg=background, fg=foreground, relief="ridge")
    catris.grid(row=2, column=1, sticky="W")

    global cattop
    cattop = tk.Radiobutton(categ, text="top", variable=catvar, value=6, command=catsel, bg=background, fg=foreground, relief="ridge")
    #cattop.grid(row=2, column=2, sticky="W")
    cattop.grid(row=1, column=1, sticky="W")

    # selects appropriate button to show which one is active
    if progvars.category == 'controversial':
        print("Current selected category is " + progvars.category)
        catcont.select()
    # elif progvars.category == 'gilded':
    #     print("Current selected category is " + progvars.category)
    #     catgil.select()
    elif progvars.category == 'hot':
        print("Current selected category is " + progvars.category)
        cathot.select()
    elif progvars.category == 'new':
        print("Current selected category is " + progvars.category)
        catnew.select()
    elif progvars.category == 'rising':
        print("Current selected category is " + progvars.category)
        catris.select()
    elif progvars.category == 'top':
        print("Current selected category is " + progvars.category)
        cattop.select()

    #close expanded view
    global butshr
    butshr = ttk.Button(main, text="<", command=winshr)
    butshr.grid(row=1, column=1, sticky="W")

#setlight and setdark could maybe be integrated into one function.
def setlight():
    modules.readconf()
    if progvars.seltheme != "light":
        progvars.seltheme = "light"
        print("Setting " + progvars.seltheme + " theme")
        changetheme()
    else:
        print("Program is already using the " + progvars.seltheme + " theme")

def setdark():
    modules.readconf()
    if progvars.seltheme != "dark":
        progvars.seltheme = "dark"
        print("Setting " + progvars.seltheme + " theme")
        changetheme()
    else:
        print("Program is already using the " + progvars.seltheme + " theme")

#code pulled out of setlight and setdark in order to avoid huge blocks of duplicate code
def changetheme():
    modules.writeconf()
    root.destroy()
    try:
        subprocess.Popen('grab-reddit-gui', shell=True, text=True)
    except:
        subprocess.Popen('python3 gui.py', shell=True, text=True)

#shrinks the window back to original size
def winshr():
    #destroy extended view frame & everything in it
    extend.destroy()
    #recreate button to expand window
    global butexp
    butexp = ttk.Button(main, text=">", command=winexp)
    butexp.grid(row=1, column=1, sticky="W")

def dirdialog():
    modules.readconf()
    #explicitly assign path variable, otherwise it complains about the variable being called before assignment
    progvars.path = tk.filedialog.askdirectory(parent=root, initialdir=progvars.path, title="Download directory")
    modules.writeconf()
    global pathlab
    pathlab.destroy()
    pathlab = ttk.Label(varfr, text="Path: " + progvars.path)
    pathlab.grid(row=0, column=1, sticky="W")

def main():
    ############
    #Initialise#
    ############

    global pathdef
    #choose default path
    pathdef = str(os.path.join(Path.home(), "Downloads", "grab-bot"))

    #make sure the config file is present and correct
    try:
        progvars.config.read('grab.ini')
        progvars.lim = progvars.config['CONFIG']['limit']
        progvars.category = progvars.config['CONFIG']['category']
        progvars.config['CONFIG']['subs']
        progvars.path = progvars.config['CONFIG']['path']
        progvars.seltheme = progvars.config['CONFIG']['theme']
        print("Config is present and correct")
    except KeyError:
        progvars.config['CONFIG'] = {'limit': '10',
                            'category': 'hot',
                            'subs': '.empty.;',
                            'path': pathdef,
                            'theme': 'light'}
        with open('grab.ini', 'w') as configfile:
            progvars.config.write(configfile)
        print("Created config or fixed missing options")

    #read config
    modules.readconf()

    global root
    #main window
    root = tk.Tk()
    root.title("grab - Reddit download bot")

    global theme
    #define theme
    theme = ttk.Style()

    global background
    global foreground
    print("Using " + progvars.seltheme + " theme")
    if progvars.seltheme == "light":
        root.configure(bg="#ffffff")
        #light theme
        theme.theme_create('light')
        theme.theme_settings('light', {
            "TFrame": {
                "configure": {
                    "background": ["#ffffff"],
                    "foreground": ["#1e1e1e"]
                }
            },
            "TLabel": {
                "configure": {
                    "background": ["#ffffff"],
                    "foreground": ["#1e1e1e"]
                }
            },
            "TButton": {
                "configure": {
                    "relief": ["ridge"],
                    "padding": [8, 1],
                    "background": ["#dedede"],
                    "foreground": ["#1e1e1e"]
                },
                "map": {
                    "relief": [("pressed", "sunken")],
                    "background": [("active", "#f0f0f0")]
                }
            },
            "TEntry": {

            }
        })
        #set background and foreground
        background = "#ffffff"
        foreground = "#1e1e1e"
        #use light theme
        theme.theme_use('light')

    elif progvars.seltheme == "dark":
        root.configure(bg="#000000")
        #dark theme
        theme.theme_create('dark')
        theme.theme_settings('dark', {
            "TFrame": {
                "configure": {
                    "background": ["#000000"],
                    "foreground": ["#e1e1e1"]
                }
            },
            "TLabel": {
                "configure": {
                    "background": ["#000000"],
                    "foreground": ["#e1e1e1"]
                }
            },
            "TButton": {
                "configure": {
                    "relief": ["ridge"],
                    "padding": [8, 1],
                    "background": ["#212121"],
                    "foreground": ["#e1e1e1"]
                },
                "map": {
                    "relief": [("pressed", "sunken")],
                    "background": [("active", "#0f0f0f")]
                }
            },
            "TEntry": {

            }
        })
        #set background and foreground
        #problems with visibility of selection from radiobuttons and spinbox up/down arrows
        #change color radiobuttons have when the mouse is hovering over them.
        background = "#000000"
        foreground = "#e1e1e1"
        #use dark theme
        theme.theme_use('dark')
    else:
        print("Sorry, this theme could not be found")

    global main
    #main frame
    main = ttk.Frame(root)
    main.grid(row=0, column=0, sticky="W", padx=5, pady=5)

    global normal
    #frame for listbox and main buttons
    normal = ttk.Frame(main)
    normal.grid(row=0, column=0, sticky="W")

    global var
    #subs label
    ttk.Label(normal, text="Active subreddits:", font="none 16").grid(row=0, column=0)
    var = tk.StringVar()

    global cnnsublist
    global labelsublist
    #change the list of subs to fit
    cnnsublist = progvars.sublist.replace(';', '\n')
    progvars.sublf = progvars.sublist.split(";")
    labelsublist = [x for x in progvars.sublf if x]

    global lbox
    #multiple doesn't work with my method
    #lbox = tk.Listbox(window, listvariable=var, selectmode="multiple", width=24)
    #create a listbox with the proper theme
    lbox = tk.Listbox(normal, listvariable=var, selectmode="single", width=26, bg=background, fg=foreground)

    #insert subreddits into listbox
    for y in range(len(labelsublist)):
        lbox.insert('end', labelsublist[y])
    lbox.grid(row=2, column=0, sticky="W")

    #add new subreddits
    global butaddsub
    butaddsub = ttk.Button(normal, text="+", command=addfr)
    butaddsub.grid(row=1, column=0, sticky="W")

    #remove subreddits
    global butremsub
    butremsub = ttk.Button(normal, text="-", command=listbox)
    butremsub.grid(row=1, column=0, sticky="E")

    #y += 1
    #close the program
    ttk.Button(main, text="Exit", command=stop).grid(row=1, column=0, sticky="W")

    #run the program
    ttk.Button(main, text="Run", command=run).grid(row=1, column=0, sticky="E")

    global butexp
    #expand
    butexp = ttk.Button(main, text=">", command=winexp)
    butexp.grid(row=1, column=1, sticky="W")

    root.mainloop()

if __name__ == '__main__':
    main()



#!/usr/bin/env python3
"""
@author: RealStickman
"""
import configparser
import multiprocessing
import sys
import praw
import prawcore
import requests
import os
from datetime import date
import argparse
from pathlib import Path
from PIL import Image

# import variables
import progvars

# config initialisation
progvars.config = configparser.ConfigParser()

# read the config file
def readconf():
    print("Reading Config file")
    progvars.config.read('grab.ini')
    progvars.lim = int(progvars.config['CONFIG']['limit'])
    progvars.category = progvars.config['CONFIG']['category']
    progvars.subl = progvars.config['CONFIG']['subs']
    progvars.sublist = progvars.subl.replace('.empty.', '')
    progvars.sublf = list(filter(None, progvars.sublist.split(';')))
    progvars.path = progvars.config['CONFIG']['path']
    progvars.seltheme = progvars.config['CONFIG']['theme']

# Write the configuration file
def writeconf():
    print("Writing config file")
    progvars.config['CONFIG'] = {'limit': progvars.lim,
                        'category': progvars.category,
                        'subs': progvars.sublist,
                        'path': progvars.path,
                        'theme': progvars.seltheme}
    with open('grab.ini', 'w') as configfile:
        progvars.config.write(configfile)

# Download stuff from reddit
def dl(subvar):
    # make path choosable
    pathdl = str(os.path.join(progvars.path, subvar, progvars.date))
    if not os.path.exists(pathdl):
        os.makedirs(pathdl)
    pathtxt = str(os.path.join(progvars.path, subvar))
    os.chdir(pathtxt)

    reddit = praw.Reddit(client_id="48VCokBQkKDsEg",
                         client_secret=None,
                         user_agent="grab, a Reddit download bot by /u/RealStickman_")

    # setting subreddit and variable for the first few posts in the hot category of it
    subreddit = reddit.subreddit(subvar)

    if progvars.category == 'controversial':
        print(progvars.category)
        posts = subreddit.controversial(limit=progvars.lim)
    elif progvars.category == 'gilded':
        print(progvars.category)
        posts = subreddit.gilded(limit=progvars.lim)
    elif progvars.category == 'hot':
        print(progvars.category)
        posts = subreddit.hot(limit=progvars.lim)
    elif progvars.category == 'new':
        print(progvars.category)
        posts = subreddit.new(limit=progvars.lim)
    elif progvars.category == 'rising':
        print(progvars.category)
        posts = subreddit.rising(limit=progvars.lim)
    elif progvars.category == 'top':
        print(progvars.category)
        posts = subreddit.top(limit=progvars.lim)
    else:
        print('This category is not implemented or does not exist')

    #test whether the subreddit exists
    try:
        subreddit.title
    except prawcore.exceptions.Redirect:
        print(progvars.CRED + subreddit.display_name + " is no subreddit" + progvars.CEND)
        return

    #creates downloaded.txt in the subreddit's directory
    try:
        downloaded = open("downloaded.txt")
        print("File exists")
        print("Downloading from " + subreddit.display_name)
    except IOError:
        print("Creating file")
        downloaded = open("downloaded.txt", "w")
        downloaded.write("")
        print("Downloading from " + subreddit.display_name)
    finally:
        downloaded.close()

    #searches the specified number of posts
    for post in posts:
        url = post.url
        filename = post.author.name + " - " + post.title + ".png"
        filetest = post.title
        downloaded = open("downloaded.txt", "r")
        string = str(downloaded.read())
        downloaded.close()
        if filetest not in string:
            print(progvars.CGRE + filename + progvars.CEND)
            reddit = requests.get(url)
            #download files from reddit
            os.chdir(pathdl)
            try:
                with open(filename, "wb") as file:
                    file.write(reddit.content)
                try:
                    Image.open(filename)
                except:
                    os.remove(filename)
                    print("Removed " + filename + ", because it is not an image.")
            except IOError:
                print("Couldn't find any picture, skipping.")
            #appends the filenames
            os.chdir(pathtxt)
            with open("downloaded.txt", "a") as downloaded:
                downloaded.write(post.title)
                downloaded.write(" ")
        else:
            print(progvars.CYEL + filename + " is already present in downloaded.txt" + progvars.CEND)

def multiprocdl():
    # number of subreddits
    progvars.numsubs = len(progvars.sublf)
    #creates a pool of processes
    try:
        p = multiprocessing.Pool(progvars.numsubs)
        #processes are started with the arguments contained in the list
        p.map(dl, progvars.sublf)
    except ValueError:
        print(progvars.CRED + "Please specify a subreddit." + progvars.CEND)
    #exit(0)



#!/usr/bin/env python3
"""
@author: RealStickman
"""
from datetime import date
import configparser

#colors for terminal output
CRED = '\033[91m'
CYEL = '\033[33m'
CGRE = '\033[92m'
CEND = '\033[0m'

# variable for configparser
config = ''

# limit
lim = ''
# category
category = ''
# array as read from the config. Used only after first creating the config.
subl = ''
# removed .empty. from subl. This will be written into the config file again.
sublist = ''
# splitting the sublist on ;
sublf = ''
# download path
path = ''
# selected theme for gui
seltheme = ''

# used in download function
# date
date = str(date.today())

# numsubs
numsubs = ''

# scheduled time
schedtime = 1



#!/usr/bin/env python3

import os
import praw
import prawcore
from datetime import date
import requests
import json
import re
from multiprocessing.pool import ThreadPool
from functools import partial

# import datetime
date = str(date.today())

# terminal colors
CRED = '\033[91m'
CYEL = '\033[33m'
CGRE = '\033[92m'
CEND = '\033[0m'

# read config options
def readconf():
    with open("grab-config.ini", "r") as conffile:
        jsonvars = conffile.read()
    progvars = json.loads(jsonvars)
    lim = progvars["limit"]
    path = progvars["path"]
    category = progvars["category"]
    sublist = progvars["sublist"]
    return lim, path, category, sublist

# write config options to json
def writeconf(lim, path, category, sublist):
    progvars = {"limit": lim,
                "path": path,
                "category": category,
                "sublist": sublist}
    # import json
    jsonvars = json.dumps(progvars)
    with open("grab-config.ini", "w") as conffile:
        conffile.write(jsonvars)

def download(lim, path, category, subvar):
    # import os
    pathdl = str(os.path.join(path, subvar, date))
    if not os.path.exists(pathdl):
        os.makedirs(pathdl)
    # go into download directory
    pathroot = str(os.path.join(path, subvar))
    os.chdir(pathroot)

    # create reddit instance
    # import praw
    reddit = praw.Reddit(client_id="48VCokBQkKDsEg",
                         client_secret=None,
                         user_agent="grab, a Reddit picture download bot by u/RealStickman_")

    # set subreddit
    subreddit = reddit.subreddit(subvar)

    # test existence of subreddit
    try:
        subreddit.title
    # import prawcore
    except prawcore.exceptions.Redirect:
        print(subreddit.display_name + " is no subreddit")
        return

    # TODO implement multiple ways of sorting here
    if category == "hot":
        submissions = subreddit.hot(limit=lim)
    elif category == "top":
        submissions = subreddit.top(limit=lim)
    elif category == "new":
        submissions = subreddit.new(limit=lim)
    else:
        print("Invalid category")


    # REVIEW
    # creates downloaded.txt in the subreddit's directory
    try:
        downloaded = open("downloaded.txt")
        print("File exists")
        print("Downloading from " + subreddit.display_name)
    except IOError:
        print("Creating file")
        downloaded = open("downloaded.txt", "w")
        downloaded.write("")
        print("Downloading from " + subreddit.display_name)
    finally:
        downloaded.close()

    # go through the posts
    # NOTE distinguishes text-posts from image posts
    # https://praw.readthedocs.io/en/latest/code_overview/models/submission.html
    for submission in submissions:
        # checks if the submission contains text. If it does, it should be skipped
        if not submission.selftext:
            # if the submission does not directly link to an image, it is discarded
            if "i." in submission.url:
                url = submission.url

                # check variables before passing them to regex
                # otherwise the program fails
                try:
                    author = submission.author.name
                except AttributeError:
                    author = "[deleted]"
                try:
                    title = submission.title
                except AttributeError:
                    title = "No title provided"

                # import re
                # NOTE removes illegal characters from filenames
                # Windows illegal characters https://gist.github.com/doctaphred/d01d05291546186941e1b7ddc02034d3
                filename = re.sub("<|>|:|\"|\/|\\|\||\?|\*", " ", author + " - " + title + ".png")
                # open the "downloaded" file and check if this submission exists already
                downloaded = open("downloaded.txt", "r")
                string = str(downloaded.read())
                downloaded.close()
                # do this if the post does not already exist in the downloaded file
                if url not in string:
                    print(CGRE + filename + CEND)
                    # import requests
                    # download the file
                    post = requests.get(url)
                    os.chdir(pathdl)
                    # write image to file
                    with open(filename, "wb") as file:
                        file.write(post.content)
                    # go back to subreddit folder
                    os.chdir(pathroot)
                    # add url to downloaded.txt
                    with open("downloaded.txt", "a") as downloaded:
                        downloaded.write(url)
                        downloaded.write(";")
                else:
                    print(CYEL + filename + " is already present in downloaded.txt" + CEND)

# NOTE look at multithreaded execution in the docs
# https://praw.readthedocs.io/en/latest/getting_started/multiple_instances.html#multiple-threads
# import multiprocessing.pool
def multiprocdownload(lim, path, category, sublist):
    # import functools
    downloadpart = partial(download, lim, path, category)
    # using 4 processes, to not stay far away from reddit rate limiting
    with ThreadPool(processes=4) as pool:
        # NOTE this has to be the last argument
        pool.map(downloadpart, sublist)

def main():
    lim, path, category, sublist = readconf()
    if len(sublist) == 0:
        print(CRED + "Please provide at least one subreddit" + CEND)
        exit(1)
    multiprocdownload(lim, path, category, sublist)
    exit(0)

if __name__ == "__main___":
    main()



#!/usr/bin/env python3

import newgrabmod
import argparse
import sys
import praw
import prawcore
import os
from pathlib import Path

# terminal colors
CRED = '\033[91m'
CYEL = '\033[33m'
CGRE = '\033[92m'
CEND = '\033[0m'

# testing config file
try:
    lim, path, category, sublist = newgrabmod.readconf()
except (FileNotFoundError, KeyError):
    print("Creating default config file")
    lim = 10
    # import os
    # import pathlib
    path = str(os.path.join(Path.home(), "Downloads", "grab-download"))
    category = "hot"
    sublist = []
    newgrabmod.writeconf(lim, path, category, sublist)

# read config file and get all variables
lim, path, category, sublist = newgrabmod.readconf()

# from here we can handle keyboard interrupts
try:
    # argparser
    # NOTE the arguments stuff was copied from the old program
    # import argparse
    parser = argparse.ArgumentParser(description='CLI-options for grab.py.', formatter_class=argparse.RawDescriptionHelpFormatter)
    arggroup = parser.add_argument_group(title='information options',
    description =
    '''-a, --add <subreddit>       Add subreddits (Multiple allowed)
    -r, --remove <subreddit>    Remove subreddits (Multiple allowed). Program will exit
    -l, --lim <limit>           Set the limit of posts
    -c, --category <category>   Set the category
    -p, --path <path>           Set the download path
    -s, --show                  Shows the configuration file. Program will exit''')

    parser.add_help

    # argument settings
    arggroup.add_argument("-a", "--add", dest="addsub", type=str, nargs='+', required=False, help=argparse.SUPPRESS)
    arggroup.add_argument("-l", "--lim", dest="limit", type=int, required=False, help=argparse.SUPPRESS)
    arggroup.add_argument("-c", "--category", dest="category", type=str, required=False, help=argparse.SUPPRESS)
    arggroup.add_argument("-p", "--path", dest="path", type=str, required=False, help=argparse.SUPPRESS)
    arggroup.add_argument("-s", "--show", action="store_true", dest="show", required=False, help=argparse.SUPPRESS)
    arggroup.add_argument("-r", "--remove", dest="removesub", type=str, nargs='+', required=False, help=argparse.SUPPRESS)

    # less to write every time
    args = parser.parse_args()

    # -s or --show
    if args.show:
        lim, path, category, sublist = newgrabmod.readconf()
        # create a comma separated string of subreddits
        substr = ", ".join(sublist)
        # print everything
        print("Subreddits: " + substr,
              "Limit: " + str(lim),
              "Path: " + path,
              "Category: " + category,
              sep="\n")
        # import sys
        sys.exit(0)

    # REVIEW
    # -r or --remove
    if args.removesub:
        for sub in args.removesub:
            # compare lowercase to prevent annoying issues with upper/lowercase
            for i, subvar in enumerate(sublist):
                if sub.lower() == subvar.lower():
                    print("Removed " + subvar)
                    del sublist[i]
        # save changes
        newgrabmod.writeconf(lim, path, category, sublist)
        sys.exit(0)

    # REVIEW
    # -a or --add
    if args.addsub:
        # compare lowercase to prevent doubles
        # e.g. re_zero while Re_Zero is already present
        sublist_lower = [item.lower() for item in sublist]
        addsub = [item.lower() for item in args.addsub]
        # diff the two lists
        subnew = [x for x in args.addsub if x not in sublist_lower]
        for sub in subnew:
            try:
                # import praw
                reddit = praw.Reddit(client_id="48VCokBQkKDsEg",
                                     client_secret=None,
                                     user_agent="grab, a Reddit picture download bot by u/RealStickman_")

                # this triggers the exception if it can't be found
                subtitle = reddit.subreddit(sub).title
                # to prevent inconsistencies between the reddit URL and this program, use the url name
                suburl = reddit.subreddit(sub).url
                # cut the "/r/" and trailing "/"
                subname = suburl[3:-1]

                # add to array
                sublist.append(subname)
                print(CGRE + "Added " + subname + CEND)
            # import prawcore
            except prawcore.exceptions.Redirect:
                print(CRED + sub + " does not exist" + CEND)
        """
        for sub in args.addsub:
            # this is needed, so you can start adding subs.
            if len(sublist) == 0:
                try:
                    # import praw
                    reddit = praw.Reddit(client_id="48VCokBQkKDsEg",
                                         client_secret=None,
                                         user_agent="grab, a Reddit picture download bot by u/RealStickman_")

                    # this triggers the exception if it can't be found
                    subtitle = reddit.subreddit(sub).title
                    # to prevent inconsistencies between the reddit URL and this program, use the url name
                    suburl = reddit.subreddit(sub).url
                    # cut the "/r/" and trailing "/"
                    subname = suburl[3:-1]

                    # add to array
                    sublist.append(subname)
                    print(CGRE + "Added " + subname + CEND)
                    sublist_lower = lowsublist(sublist)
                # import prawcore
                except prawcore.exceptions.Redirect:
                    print(CRED + sub + " does not exist" + CEND)
            else:
                for subvar in sublist_lower:
                    if sub in sublist_lower:
                        print(CYEL + subvar + " has already been added" + CEND)
                    else:
                        try:
                            # import praw
                            reddit = praw.Reddit(client_id="48VCokBQkKDsEg",
                                                 client_secret=None,
                                                 user_agent="grab, a Reddit picture download bot by u/RealStickman_")

                            # this triggers the exception if it can't be found
                            subtitle = reddit.subreddit(sub).title
                            # to prevent inconsistencies between the reddit URL and this program, use the url name
                            suburl = reddit.subreddit(sub).url
                            # cut the "/r/" and trailing "/"
                            subname = suburl[3:-1]

                            # add to array
                            sublist.append(subname)
                            print(CGRE + "Added " + subname + CEND)
                            sublist_lower = lowsublist(sublist)
                            break
                        # import prawcore
                        except prawcore.exceptions.Redirect:
                            print(CRED + sub + " does not exist" + CEND)
        """

    # -l or --lim
    if args.limit:
        lim = args.limit

    # -c or --category
    if args.category:
        category = args.category

    # -p or --path
    if args.path:
        path = args.path

    # write potential changes to the config file
    newgrabmod.writeconf(lim, path, category, sublist)

    def main():
        newgrabmod.main()

    if __name__ == "__main__":
        main()
except KeyboardInterrupt:
    newgrabmod.writeconf(lim, path, category, sublist)
    print("Exiting program")
    # return default linux SIGINT code
    exit(130)



#!/usr/bin/env python3
"""
@author: RealStickman
"""
import time

# import modules
import modules

import progvars

# FIXME Does not appear to work on windows
def main():
    # This line translates the minutes entered in schedtime into seconds
    schedtimes = progvars.schedtime * 60
    
    starttime = time.time()
    while True:
        # The appropriate functions are called
        modules.readconf()
        modules.multiprocdl()
        # Sleep for the right amount of time
        time.sleep(schedtimes - ((time.time() - starttime) % schedtimes))



#!/usr/bin/env python3
"""
@author: RealStickman
"""
import configparser
import multiprocessing
import sys
import praw
import prawcore
import requests
import os
from datetime import date
import argparse
from pathlib import Path
from PIL import Image

# variables
import progvars
# modules
import modules

###########
#variables#
###########

#colors for terminal output
CRED = '\033[91m'
CYEL = '\033[33m'
CGRE = '\033[92m'
CEND = '\033[0m'
#choose default path
pathdef = str(os.path.join(Path.home(), "Downloads", "grab-bot"))

############
#Initialise#
############

try:
    progvars.config.read('grab.ini')
    progvars.config['CONFIG']['category']
except KeyError:
    progvars.config['CONFIG'] = {'limit': '10',
                        'category': 'hot',
                        'subs': '.empty.;',
                        'path': pathdef,
                        'theme': 'light'}
    with open('grab.ini', 'w') as configfile:
        progvars.config.write(configfile)
    print("Created default configuration file")
finally:
    modules.readconf()

#################
#Argument parser#
#################

parser = argparse.ArgumentParser(description='CLI-options for grab.py.', formatter_class=argparse.RawDescriptionHelpFormatter)
g = parser.add_argument_group(title='information options',
description =
'''-s, --sub <subreddit>        Add subreddits (Multiple allowed)
-r, --remove <subreddit>    Remove subreddits (Multiple allowed). Program will exit
-l, --lim <limit>           Set the limit of posts  
-c, --category <category>   Set the category
-p, --path <path>           Set the download path
-v, --variables             Shows the configuration file. Program will exit''')

parser.add_help
g.add_argument("-s", "--sub", dest="subreddit", type=str, nargs='+', required=False, help=argparse.SUPPRESS)
g.add_argument("-l", "--lim", dest="limit", type=int, required=False, help=argparse.SUPPRESS)
g.add_argument("-c", "--category", dest="category", type=str, required=False, help=argparse.SUPPRESS)
g.add_argument("-p", "--path", dest="path", type=str, required=False, help=argparse.SUPPRESS)
g.add_argument("-v", "--variables", action="store_true", dest="variables", required=False, help=argparse.SUPPRESS)
g.add_argument("-r", "--remove", dest="removesub", type=str, nargs='+', required=False, help=argparse.SUPPRESS)

args = parser.parse_args()
# print(args.subreddits)

# NOTE Make output of this more beautiful instead of just pasting the config. (Especially subreddits section)
argvars = args.variables
# if this is set, show the config file and exit
if (argvars == True):
    try:
        conffile = open("grab.ini")
        readbackfile = conffile.read()
        print(readbackfile)
        conffile.close()
        exitprog = True
    except:
        print("Could not read grab.ini")
    sys.exit(0)

# Argument for removing subs. Quits after execution.
# TODO Make case insensitive
argrem = args.removesub
if argrem is not None:
    returnsubs = "Removed "
    for s in range(len(argrem)):
        progvars.sublist = progvars.sublist.replace(";" + argrem[s], '')
        returnsubs = returnsubs + argrem[s] + ", "
    modules.writeconf()
    print(returnsubs)
    sys.exit(0)

argsubs = args.subreddit
#try getting an argument and add that to the list
if argsubs is not None:
    for s in range(len(argsubs)):
        # check if the subreddit is already in the array
        if (argsubs[s] in progvars.sublist):
            print(argsubs[s] + " has already been added")
        else:
            try:
                #reddit stuff
                reddit = praw.Reddit(client_id="48VCokBQkKDsEg",
                                     client_secret=None,
                                     user_agent="grab, a Reddit download bot by /u/RealStickman_")

                subreddit = reddit.subreddit(argsubs[s])
                subreddit.title

                #add entered subreddit
                progvars.sublist += subreddit.display_name + ";"
                #that's were we're gonna put it
                print("Added " + subreddit.display_name)
            #if the subreddit can't be found
            except prawcore.exceptions.Redirect:
                print("Wrong Subreddit", subreddit.display_name + " does not exist")
else:
    print("No subreddit given")

arglim = args.limit
#try getting a limit
if arglim is not None:
    progvars.lim = args.limit
else:
    print("No limit given")

argcategory = args.category
#try getting a category
if argcategory is not None:
    progvars.category = args.category
else:
    print("No category given")

argpath = args.path
#try getting a path
if argpath is not None:
    progvars.path = args.path
else:
    print("No path given")

modules.writeconf()

modules.readconf()

#number of subreddits that have been specified in the config
#numsubs = len(progvars.sublf)

def main():
    #global numsubs
    #global dl
    # number of subreddits
    progvars.numsubs = len(progvars.sublf)
    #creates a pool of processes
    try:
        p = multiprocessing.Pool(progvars.numsubs)
        #processes are started with the arguments contained in the list
        p.map(modules.dl, progvars.sublf)
    except ValueError:
        print(progvars.CRED + "Please specify a subreddit." + progvars.CEND)
    exit(0)

if __name__ == '__main__':
    #main()
    modules.multiprocdl()
